#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "non_gui_defs.h"
#include "hound_functions.h"

/* generate the 20 dogs of a cpu kennel or 15 of the
   home kennel at the beginning of the game */
gint generate_batch(gchar names[][19], gint kennel_id)
{
    gint i,j,k;

    /* number of dogs on each of the 5 levels */
    gint number_of_dogs[6];

    /* number of dogs in kennel, 12 or 20 */
    gint dogs_in_kennel;

    /* determine how many dogs on the various levels
       are in the kennel */
    for(i=0;i<6;i++)
	number_of_dogs[i] = (gint)rint( rnd(2,6) );

    if(kennel_id == 0)
    {
	dogs_in_kennel = 15;
	for(i=15;i<20;i++)
	    dogs[i].age = -1;
    }
    else
	dogs_in_kennel = 20;

    /* since we want to have exactly 20 dogs in each cpu-kennel,
       and 15 in the first kennel, we substract or add 1 to each
       level in a cycle until we've reached the desired value. */
    
    i = (gint)rint( rnd(0,5) );
    while( (gint)sum(number_of_dogs,0,5) != dogs_in_kennel )
    {
	if( (gint)sum(number_of_dogs,0,5) > dogs_in_kennel )
	{
	    if( number_of_dogs[i] != 0 )
		number_of_dogs[i]--;
	}
	else if( (gint)sum(number_of_dogs,0,5) < dogs_in_kennel )
	    number_of_dogs[i]++;

	i = (i + 1) % 6;
    }
    
    /* fill in data for each dog */
    k=0;
    for(i=0;i<6;i++)
	for(j=0;j<number_of_dogs[i];j++)
	{	    
	    /* fill in name, age, potential level, gender,
	       fitness and race statistics.
	       if startup is '1' it's a randomly generated dog,
	       otherwise a birth.

	       potential level signifies the level the dog should have
	       at the peak of its career, i.e. at the age of 3. the real
	       level results from the skill values multiplied by the
	       fitness value */
	    fill_in_general_data(&dogs[ kennel_id * 20 + k ], names, i, 1);
	    
	    /* parents are not present in the game, so
	       we assign unique id's that don't belong to a real dog: */
	    dogs[ kennel_id * 20 + k ].mother_id = - kennel_id * 20 + k - 1;
	    dogs[ kennel_id * 20 + k ].father_id = 2 * (- kennel_id * 20 + k - 1);

	    /* the names don't have to be unique */
	    strcpy( dogs[ kennel_id * 20 + k ].mother_name, "NAME" );
	    strcpy( dogs[ kennel_id * 20 + k ].father_name, "NAME" );

	    /* generate speed, stamina and ancestor values.
	       they all depend on potential level;
	       calculate the real level afterwards from the skill
	       values */
	    generate_skills(&dogs[ kennel_id * 20 + k ], i);
	    
	    /* calculate a dog's level either from its skill values
	       (at startup) or using the results of the last 5 races */
	    dogs[ kennel_id * 20 + k ].level =
		calculate_level_skills(dogs[ kennel_id * 20 + k ]);

	    k++;
	}



    return 0; 
}

/* generate speed etc. values at the beginning of a new game */
gint generate_skills(dog *Dog, gint level)
{
    gfloat speed, stamina;
    gfloat ancestor_speed, ancestor_stamina;
    gfloat factor_range;

    /* flip a cogint to find out whether speed or
       stamina is the dog's main talent */
    if(rnd(0,1) < 0.5)
    {
	speed = gaussvt( level + 1, level + 2, level + 4.9, 9.9 );
	stamina = speed * rnd(0.5, 1);
	if(stamina >= 9.9)
	    stamina = 9.9;
    }
    else
    {
	stamina = gaussvt( level + 1, level + 2, level + 4.9, 9.9 );
	speed = stamina * rnd(0.5, 1);
	if(speed >= 9.9)
	    speed = 9.9;
    }

    /* generate both values. 
       since Dog->speed/stamina is an array, we turn the
       values from above into a pseudo-genetic code
       of 0's and 1's */
    generate_code(Dog->speed, speed);
    generate_code(Dog->stamina, stamina);

    /* it's kinda reasonable to have ancestor values
       in the vicinity of the speed/stamina values.

       on the other hand, the original value shouldn't be
       farther away from the ancestor value than from 1 */
    if( 9.9 / speed - 1 < 0.3)
	factor_range = 9.9 / speed - 1;
    else
	factor_range = 0.3;
    ancestor_speed = speed * rnd( 1 - factor_range, 1 + factor_range );

    if( 9.9 / speed - 1 < 0.3)
	factor_range = 9.9 / speed - 1;
    else
	factor_range = 0.3;
    ancestor_stamina = stamina * rnd( 1 - factor_range, 1 + factor_range );
    
    generate_code(Dog->ancestor_speed, ancestor_speed);
    generate_code(Dog->ancestor_stamina, ancestor_stamina);
    
    return 0;
}

/* fill in name, age, potential level, gender, fitness and race statistics.
   if type is '1' it's a randomly generated dog at startup, if it's 11
   it's a randomly generated dog replacing an old dog in a cpu-kennel;
   if it's 0 it's  a birth.

   potential level signifies the level the dog should have at the peak
   of its career, i.e. at the age of 3. the real level results from the
   skill values multiplied by the fitness value */
gint fill_in_general_data(dog *Dog, gchar names[][19], gint level, gint type)
{
    gint i;
    gint random;
    
    /* read random name from buffer and assign age.
       if this is startup, 'random_dog' is 1 and the
       buffer contains
       if 'random_dog' is 11 it's the random generation of
       a new dog in a cpu kennel; if 'random_dog' is 0
       it's a birth. in the latter cases the buffer
       contains only 500 names */
    if(type % 10 == 1)
    {
	if(type == 1)
	{
	    random = (gint)rint( rnd(0, 8223) );
	    /* age's between 1 year and 5 years */
	    Dog->age = (gint)rint( rnd(12,60) );	    
	}
	else
	{
	    /* new dogs are always young */
	    Dog->age = 12;
	    random = (gint)rint( rnd(0, 499) );
	}	

	strcpy(Dog->name,names[random]);
    }
    else
    {
	Dog->age = 0;
	strcpy(Dog->name,names[	(gint)rint( rnd(0, 499) ) ]);
    }

    Dog->level = level;

    /* assign gender by coin throw */
    if(rnd(0,1) < 0.5)
	Dog->gender = 0;
    else
	Dog->gender = 1;

    /* set status to normal if dog,
       set cycle of bitches randomly */
    if(Dog->gender == 0)
	Dog->status = 19;
    else
	Dog->status =  (gint)rint( rnd(0,11) );

    /* the fitness factor depends on 'age'. status depends
       on injury, cycle etc. */
    calculate_fitness_status(Dog, type % 10);
    
    /* fertility depends on age */
    Dog->fertility = calculate_fertility(*Dog);

    for(i=0;i<6;i++)
    {
	if(i < 5)
	    Dog->last_races[i * 2] = 15;
	Dog->best_times[i] = 100;
	Dog->races[i] = 0;
    }

    return 0;
}

/* calculate fitness of a dog depending on age, cycle (if 
   it's a she) and whether or not this is a random value-generation.

   type = 1: random dog generation, dog is grown-up
   type = 0: assignment of fitness to a newborn puppie
             or a dog younger than 12
   type = 2: semi-monthyl recalculation

   fitness increases like c * x^3 (i.e. first slowly, then quite
   steeply) from 0 at 0 to 0.6 at 12; it's then
   continues parabola-like, reaches a peak of 0.999 at 36 and
   decreases to 0.6 at 60 and 0 at appr. 74.

   there's also a random component in the monthly recalculation:
   rarely, when a dog is younger than 3 (years), fitness does not increase;
   or when older than 3, it does not decrease. */
gint calculate_fitness_status(dog *Dog, gint type)
{
    gfloat delta_fitness;

    /* first calculate fitness which
       is then multiplied by 'speed' and 'stamina'
       to result in the current skill values of the dog.
       fitness of a newborn puppy is of course 0 */
    if( type != 2)
	Dog->fitness = (type == 1) ?
	    -0.0006927 * pow(Dog->age - 36, 2) + 0.999 : 0;
    
    /* if this is monthly recalculation, compute the difference
       between the old fitness (at age 'Dog->age') and new fitness
       (at age 'Dog->age + 0.5') */
    else
    {
	delta_fitness = -0.0006927 * ( pow( Dog->age - 35.5, 2) -
				       pow( Dog->age - 36, 2) );

	/* now there's a 10% chance that amelioration or
	   deterioration is retarded with grown-ups : */
	if( rnd(0,1) < 0.9 && Dog->age > 12)
	    Dog->fitness += delta_fitness;
	if(Dog->age < 12)
	    Dog->fitness = 0.000347 * pow(Dog->age + 0.5, 3);
    }
    
    /* from age 74 onwards a dog has fitness 0;
       negative values are not allowed */
    Dog->fitness = (Dog->fitness < 0) ? 0 : Dog->fitness;

    /* calculate new status */
    /* duration of injury diminishes */
    if(19 < Dog->status && Dog->status < 28)
	Dog->status--;
    else if(Dog->gender == 1)
    {
	/* the cycle of bitches moves on */
	if(Dog->status < 20)
	    Dog->status = (Dog->status + 1) % 12;
	/* recovery from birth is completed and
	   cycle reset to somewhere in the beginning */
	else if(Dog->status == 50)
	    Dog->status = (gint)rint( rnd(1,4) );
	/* pseudo pregnancy is uncovered */
	else if(Dog->status == 4001)
	    Dog->status = 1;
	/* the date of birth comes closer */
	else
	    Dog->status -= 1000;	
    }
    
    return 0;
}

/* calculate fertility for a dog depending on age.

   fertility increases linearly from 0.5 to 1 between 12 and 24 months,
   is constantly 1 until the age of 48 and decreases linearly to 0 at the
   age of 72.

   fertility of course plays a role when interbreeding */
gfloat calculate_fertility(dog Dog)
{
    if(Dog.age < 12 || Dog.age > 71)
	return 0;
    else if(Dog.age < 24)
	return 0.04167 * Dog.age;
    else if(Dog.age < 49)
	return 1;
    else
	return - 0.04167 * Dog.age + 3; 
}

/* calculate a dog's level from its skill values */
gint calculate_level_skills(dog Dog)
{
    gfloat maximal_value;
    gint new_level;
    
    /* see which of the two skills is higher */
    maximal_value = ( sum(Dog.speed,0,98) > sum(Dog.stamina,0,98) ) ?
	sum(Dog.speed,0,98) : sum(Dog.stamina,0,98);

    /* the real value depends on the fitness value */
    maximal_value *= Dog.fitness;
	
    /* assign level */
    if(maximal_value <=  30)
	new_level = 0;
    else if(maximal_value <=  40)
	new_level = 1;
    else if(maximal_value <=  50)
	new_level = 2;
    else if(maximal_value <=  60)
	new_level = 3;
    else if(maximal_value <=  70)
	new_level = 4;
    else
	new_level = 5;
    
    return new_level;
}

/* monthly recalculation of a dog's level using
   its results of the last five races */	
gint recalculate_level(dog Dog)
{
    gint points=0,i;

    /* the dog should have participated in some of
       the races; otherwise we recalculate the level
       by his skills */
    if(Dog.races[0] < (gfloat)month * 0.5)
	return calculate_level_skills(Dog);

    /* give points for places:
       5 to 1 for 1st to 5th place, -1 to -5 for 6th to 10th place.

       places are stored in even entries in 'last_races';
       times are correspondingly stored in odd entries */
    for(i=0;i<5;i++)
	if(Dog.last_races[i*2] < 6)
	    points += - Dog.last_races[i*2] + 6;
	else if(Dog.last_races[i*2] < 11)
	    points += - Dog.last_races[i*2] + 5;
    /* if there were no previous races (which is noted by 
       last_races = 15), ignore points */
	else
	    points += 0;
    
    /* since we give higher priority to places,
       we're done if 'points' is extraordinary enough.
       the cutoff of 14 points is equivalent to 5 3rd places,
       whereas -14 is equivalent to 5 7th places.
    
       if the dog's level improves we note it by adding 201
       to it; 200 we'll remove later; deterioration is noted
       by adding 99.
       the reason behind this is for once, that we recognize in other
       functions a level change and display it to the user; and furthermore
       in the semi-monthly evaluation we skip the level recalculation three
       times after a level change (otherwise bad or good race results 
       lead to multiple level changes; e.g. if a dog is 1st three times
       he has 18 points in three races and his level improves; if he's 7th in
       his 4th race he's got 16 points which would again qualify him for
       level change) by substracting 10 from the level value */
    if(points > 14 && Dog.level < 5)
	return Dog.level + 201;
    else if(points < -14 && Dog.level > 0)
	return Dog.level + 99;

    /* if there were no exceptional results, it could have been
       because of unusually strong opponents.
       now we check whether the five race times were special*/
    else
	return recalculate_level_times(Dog);
    
}

/* check whether the times in the last five races
   qualify (or condemn) the dog for a level change.

   the treshold values are empirical */
gint recalculate_level_times(dog Dog)
{
    gint i, race_length;
    gint treshold_crosses=0;
    gfloat tresholds[8][3]={{19.16, 40.39 ,63.58},
			   {18.88, 39.71, 62.18},
			   {18.60, 39.03, 60.78},
			   {18.33, 38.34, 59.34},
			   {18.05, 37.66, 57.94},
			   {17.76, 36.98, 56.55},
			   {17.38, 36.05, 54.70},
			   {17.20, 35.50, 53.60}};

    for(i=0;i<5;i++)
    {
	/* first determine the length of the race */
	if(Dog.last_races[ 2*i + 1 ] < 25)
	    race_length = 1;
	else if(Dog.last_races[ 2*i + 1 ] < 45)
	    race_length = 2;
	else
	    race_length = 3;

	/* count how many times the dog crossed the treshold
	   to an adjacent level */
	if( Dog.last_races[ 2 * i + 1 ] <
	    tresholds[Dog.level + 2][race_length - 1] &&
	    Dog.last_races[2 * i] < 10)
	    treshold_crosses++;
	else if(Dog.last_races[ 2 * i + 1 ] >
		tresholds[Dog.level][race_length - 1] &&
		Dog.last_races[2 * i] < 10)
	    treshold_crosses--;
    }


    /* if the crossings into one level outnumber the
       crossings into the opposite one 4:1 or higher,
       change level */
    if(treshold_crosses > 2 && Dog.level < 5)
	return Dog.level + 201;
    else if(treshold_crosses < -2 && Dog.level > 0)
	return Dog.level + 99;

    else
	return Dog.level;
}

/* change age, fitness and (sometimes)
   level of a dog */
gint semi_monthly_reevaluation(dog *Dog)
{
    gint i;
    gint level;
    gint puppy_info;

    /* if we aren't starting a new season, recalculate levels */
    if(month != 1 || week != 1)
    {
	/* if there's been no level change since 3 races, recalculate
	   level */
	if( Dog->level < 80 || (Dog->level > 110 && Dog->level < 180) )
	{
	    Dog->level = Dog->level % 10;
	    Dog->level = recalculate_level(*Dog);
	}
    /* otherwise 'remove a counter' */
	else
	    Dog->level -= 10;
    }
    
    /* second argument is 0 for this is no random generation */
    calculate_fitness_status(Dog, 2);
    
    Dog->age += 0.5;

    /* the date of birth for pregnant bitches may be now;
       if the puppies had no room on the puppy list (which can
       contain only 20 pups at most), return warning */
    puppy_info = calculate_birth(Dog);
    
    Dog->fertility = calculate_fertility(*Dog);

    /* if we have a cpu dog with age > 60 (older than 5 years)
       we replace it with a random younger one to keep the cpu
       kennels fresh.
       we make two exceptions: if a dog is to be the father or
       mother of the human player's puppies next semi-month,
       we've got to keep him/her for value generation. this
       information is stored in status: dogs' status is set to
       18 and bitches' to something > 99 */
    if( Dog->id > 19 && Dog->age > 60 && Dog->status != 18 &&
	Dog->status < 100)
    {
	/* remove dog from descendants', too */
	for(i=0;i<400;i++)
	    if(Dog->id == dogs[i].father_id &&
	       strcmp(Dog->father_name, dogs[i].name) == 0)
	    {
		Dog->father_id = 2 * (- Dog->id - 1);
		strcpy( Dog->father_name, "NAME" );	
	    }
	    else if(Dog->id == dogs[i].mother_id &&
		    strcmp(Dog->mother_name, dogs[i].name) == 0)
	    {
		Dog->mother_id = - Dog->id - 1;
		strcpy( Dog->mother_name, "NAME" );	
	    }
	
	level = (gint)rint( rnd(0,6) );
	if( level > 5 )
	    level = 0;

	fill_in_general_data(Dog, names_kept, level, 11);

	/* id stays the same, but parents' data is new */
	Dog->mother_id = - Dog->id - 1;
	Dog->father_id = 2 * (- Dog->id - 1);
	strcpy( Dog->mother_name, "NAME" );
	strcpy( Dog->father_name, "NAME" );	
	
	generate_skills(Dog, level);

	Dog->level = calculate_level_skills(*Dog);
    }
    
    return puppy_info;
}

gint calculate_birth(dog *Dog)
{
    gint i,j;
    gint puppy_info = 0;
    gint number_of_pups = 0;
    dog puppies[6];
    
    /* birth takes only place when status is 0xxx with xxx > 99 or
       when it's 5xxx with xxx similar (latter one is disadvantageous
       puppy distribution) */
    if( Dog->status < 100 || (Dog->status > 1000 && Dog->status < 5000) ||
	Dog->status > 6000 )
	return 0;
    else
    {
	/* find father */
	for(i=0;i<400;i++)
	    if(dogs[i].id == (Dog->status % 1000) - 100)
	    {
		number_of_pups = (gint)gaussvt(1,1,6,6);

		if(Dog->id > 19 || dogs[i].id > 19)
		{
		    /* normal cooperation with another kennel: flip
		       coin to find out who gets one pup more if there
		       is an odd number of them */
		    if(Dog->status < 1000)
		    {
			if(number_of_pups % 2 == 1)
			    number_of_pups = (rnd(0,1) < 0.5) ?
				(gint)((gfloat)number_of_pups / 2) :
				(gint)((gfloat)number_of_pups / 2) + 1;
			else
			    number_of_pups = (gint)((gfloat)number_of_pups / 2);
		    }
		    /* disadvantageous cooperation: */
		    else
		    {
			if(number_of_pups % 2 == 1)
			{
			    if(number_of_pups != 1)
				number_of_pups =
				    (gint)((gfloat)number_of_pups / 2);
			    else
				number_of_pups = 1;
			}
			else
			    number_of_pups =
				(gint)((gfloat)number_of_pups / 2) - 1;
		    }

		    /* reset father's status to normal */
		    dogs[i].status = 19;		    
		}

		/* exclude the possibility of no pup */
		if(number_of_pups == 0)
		    number_of_pups = 1;
		
		/* fill in puppies */
		mating(dogs[i], *Dog, puppies, number_of_pups);

		break;
	    }
	
	if(i == 399 && dogs[i].id != (Dog->status % 1000) - 100)
	    g_print("Error: no father found in calc_birth\n");

	

	if(20 - dogs_in_kennel(20) < number_of_pups)
	    puppy_info = 2;
	else
	    puppy_info = 1;

	/* copy puppies into the puppy list */
	for(i=0;i<number_of_pups;i++)
	    for(j=400;j<420;j++)
		if(dogs[j].age == -1)
		{
		    copy_dogs(puppies[i], &dogs[j]);
		    break;
		}
	
	/* set bitch's status to 'recovering' */
	Dog->status = 50;
    }    
    
    return puppy_info;
}
/* return the speed or stamina value that the human player
   really sees in the game */
gfloat real_value(dog Dog, gint speed)
{
    if(speed == 1)
	return sum(Dog.speed,0,98) * Dog.fitness;

    return sum(Dog.stamina,0,98) * Dog.fitness;
}

/* sort dogs of a kennel depending on an attribute */
gint sort_dogs(gint kennel_id, gint attribute)
{
    gint i;
    gint temporary_order[20];
    gfloat to_be_ordered[20];
    gint sign;
    
    if( attribute == 0 ||
	(abs(attribute) > 7 && (attribute / 10 < 2 || attribute / 10 > 4) ) )
    {
	g_print("\nSort_dogs: unknown attribute %d\n",attribute);
	return -1;
    }
    
    /* in order[420] the current ordering attribute
       for the dogs is stored ; if it's negative, the ordering
       is ascending, otherwise descending. */

    /* set reverse order if the user clicks on the same column
       by which the dogs are currently sorted */
    if( attribute == abs(order[420]) )
    {
	if( kennel_id == 0 )
	    order[420] *= -1;
	attribute = order[420];
    }
    else
	order[420] = attribute;

    sign = attribute / abs(attribute);

    if(abs(attribute) == 1)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = dogs[ kennel_id * 20 + i ].age * sign;
    else if(abs(attribute) == 2)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = (gfloat)dogs[ kennel_id * 20 + i ].gender * sign;
    else if(abs(attribute) == 3)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = (gfloat)(dogs[ kennel_id * 20 + i ].level % 10)
		* sign;
    /* speed */
    else if(abs(attribute) == 4)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = real_value(dogs[ kennel_id * 20 + i], 1) * sign;
    /* stamina */
    else if(abs(attribute) == 5)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = real_value(dogs[ kennel_id * 20 + i], 0) * sign;
    /* points */
    else if(abs(attribute) == 6)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = points_of(dogs[ kennel_id * 20 + i ]) * sign;
    /* status: injured, in heat or OK */
    else if(abs(attribute) == 7)
	for(i=0;i<20;i++)
	    to_be_ordered[i] = dogs[ kennel_id * 20 + i ].status * sign;

    /* qualification for a 1-lap race; needed when entering kennels
       in trophy races, not for the human player */
    else if(attribute == 20)
	for(i=0;i<20;i++)
	    to_be_ordered[i] =
		0.75 * real_value(dogs[ kennel_id * 20 + i], 1) +
		0.25 * real_value(dogs[ kennel_id * 20 + i], 0);
    /* 2 laps */
    else if(attribute == 30)
	for(i=0;i<20;i++)
	    to_be_ordered[i] =
		0.5 * real_value(dogs[ kennel_id * 20 + i], 1) +
		0.5 * real_value(dogs[ kennel_id * 20 + i], 0);
    /* 3 laps */
    else if(attribute == 40)
	for(i=0;i<20;i++)
	    to_be_ordered[i] =
		0.25 * real_value(dogs[ kennel_id * 20 + i], 1) +
		0.75 * real_value(dogs[ kennel_id * 20 + i], 0);    


    sort_float_array(to_be_ordered, temporary_order, 0, 19);

    for(i=0;i<20;i++)
	order[ 20 * kennel_id + i ] = temporary_order[i] + 20 * kennel_id;

    return 0;
}

/* add up the points a dog has earned based on the
   number of finishes in top 5 */
gint points_of(dog Dog)
{
    gint points = 0;
    gint i;

    /* a win is worth 6 points */
    points += Dog.races[1] * 6;
    
    /* 2nd to 5th places are worth 4 to 1 points */
    for(i=2;i<6;i++)
	points += Dog.races[i] * (6 - i);
    
    return points;
}

/* return the id of father or mother of 'Dog' */
gint find_parent_of(dog Dog, gint father)
{
    gint i;

    for(i=0;i<400;i++)
	if( (dogs[i].id == Dog.father_id &&
	     strcmp(dogs[i].name, Dog.father_name) == 0 && father == 1) ||
	    (dogs[i].id == Dog.mother_id &&
	     strcmp(dogs[i].name, Dog.mother_name) == 0 && father == 0) )
	    return i;

    return -1;
}

/* return the number of dogs in a kennel */
gint dogs_in_kennel(gint kennel_id)
{
    gint number_of_dogs=0;
    gint i;

    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
    {
	if(dogs[i].age != -1)
	    number_of_dogs++;
    }

    return number_of_dogs;
}

/* at random dog generation make up average results
   for the last 5 races so that the monthly level
   reevaluation has something to chew on */
gint make_up_last_races(dog *Dog)
{
    gint i;
    gfloat tresholds[8][3]={{19.16, 40.39 ,63.58},
			   {18.88, 39.71, 62.18},
			   {18.60, 39.03, 60.78},
			   {18.33, 38.34, 59.34},
			   {18.05, 37.66, 57.94},
			   {17.76, 36.98, 56.55},
			   {17.38, 36.05, 54.70},
			   {17.20, 35.50, 53.60}};

    for(i=0;i<5;i++)
	Dog->last_races[i * 2 + 1] =
	    tresholds[ Dog->level + 1 ][1];
    
    return 0;
}

/* compare two dogs either for trading or interbreeding.
   for young dogs we take their expected peak skill, for older ones
   their current skill when trading. 
   when mating, we take the peak skill (sensible, isn't it?) */
gint compare_dogs(dog Dog1, dog Dog2, gint trade)
{
    gfloat skills_dog1, skills_dog2;
    gfloat random;

    if(Dog1.age < 36 || trade == 0)
	skills_dog1 = sum(Dog1.speed, 0, 98) + sum(Dog1.stamina, 0, 98);
    else
	skills_dog1 = real_value(Dog1,0) + real_value(Dog1,1);

    if(Dog2.age < 36 || trade == 0)
	skills_dog2 = sum(Dog2.speed, 0, 98) + sum(Dog2.stamina, 0, 98);
    else
	skills_dog2 = real_value(Dog2,0) + real_value(Dog2,1);


    /* gauss distributed random number without cutoff */
    random = gaussvt(1, -10, 10, 1);

    /* if the two numbers are equal, the probability of
       returning 1 is 50%; if skills_dog1 > skills_dog2
       the probability is greater than 50% and vice versa */
    if(random < skills_dog1 - skills_dog2)
    {
	/* if the owner agrees, he forgets past rejections */
	options_kennel_block[trade][ (gint) ((gfloat)Dog2.id / 20) - 1 ] = 0;
	return 1;
    }
    /* the cpu-kennel may agree with interbreeding despite his dog
       being better if he gets the majority of the pups */
    else if( random < (skills_dog1 - skills_dog2) + rnd(8,18) &&
	     trade == 0 )
	return 5;
    else
    {
	/* the cpu-owner refuses; we store this fact.
	   when the number of refusals in one week reaches 3,
	   the owner refuses to trade/mate for 3 semi-months; we record
	   this by setting the value to 9 and removing 3 each semi-month */
	if( options_kennel_block[trade][ (gint) ((gfloat)Dog2.id / 20) - 1 ] < 2)
	{
	    options_kennel_block[trade][ (gint) ((gfloat)Dog2.id / 20) - 1 ]++;
	    return 2;
	}
	else
	{
	    options_kennel_block[trade][ (gint) ((gfloat)Dog2.id / 20) - 1 ] = 9;
	    return 3;
	}
    }
    
    return 0;
}

/* swap two dogs when trading is done.
 there's one thing we have to take care of:
 if one of the dogs is becoming father, then the
 bitch has stored his id in her status value, so
 we have to alter that */
gint swap_dogs(dog *Dog1, dog *Dog2)
{
    dog swapdog;
    gint id1 = Dog1->id;
    gint id2 = Dog2->id;
    gint i;

    copy_dogs(*Dog1, &swapdog);
    copy_dogs(*Dog2, Dog1);
    copy_dogs(swapdog, Dog2);

    for(i=0;i<400;i++)
    {
	if(dogs[i].status % 1000 - 100 == id1)
	    dogs[i].status = dogs[i].status -
		dogs[i].status % 1000 + id2 + 100;
	else if(dogs[i].status % 1000 - 100 == id2)
	    dogs[i].status = dogs[i].status - 
		dogs[i].status % 1000 + id1 + 100;
    }
    
    return 0;
}

/* copy dog1 to dog2 */
gint copy_dogs(dog Dog1, dog *Dog2)
{
    gint i;

    strcpy(Dog2->name, Dog1.name);
    Dog2->gender = Dog1.gender;
    Dog2->level = Dog1.level;
    Dog2->age = Dog1.age;

    for(i=0;i<99;i++)
    {
	Dog2->speed[i] = Dog1.speed[i];
	Dog2->stamina[i] = Dog1.stamina[i];
	Dog2->ancestor_speed[i] = Dog1.ancestor_speed[i];
	Dog2->ancestor_stamina[i] = Dog1.ancestor_stamina[i];
    }
    
    Dog2->fitness = Dog1.fitness;
    Dog2->status = Dog1.status;
    Dog2->fertility = Dog1.fertility;
    Dog2->mother_id = Dog1.mother_id;
    Dog2->father_id = Dog1.father_id;
    strcpy(Dog2->mother_name, Dog1.mother_name);
    strcpy(Dog2->father_name, Dog1.father_name);

    for(i=0;i<6;i++)
    {
	Dog2->races[i] = Dog1.races[i];
	Dog2->best_times[i] = Dog1.best_times[i];
    }

    for(i=0;i<10;i++)
	Dog2->last_races[i] = Dog1.last_races[i];

    return 0;
}

gint kennel_has_bitch_in_heat(gint kennel_id)
{
    gint i;
    
    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
	if(dogs[order[i]].age > 11 &&
	   dogs[order[i]].status == 0)
	    return 1;
    
    return 0;
}

/* return id of dog with the given best_time */
gint get_dog_id(gint kennel_id, gfloat time)
{
    gint i,j;

    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
    {
	for(j=0;j<3;j++)
	    if(dogs[i].best_times[j] == time)
		return i;
    }

    return -1;
}

/* compute and return the value corresponding to 
 'attribute' as described in 'sort_kennel_statistics'.
 if attribute = 200, return number of races */
gfloat kennel_best(gint kennel_id, gint attribute)
{
    gint i, j;
    gfloat best_times[3] = {200, 200, 200};
    gfloat races = 0, wins = 0, points = 0;

    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
	if(dogs[i].age != -1)
	{
	    races += dogs[i].races[0];
	    wins += dogs[i].races[1];
	    points += points_of(dogs[i]);
	    
	    for(j=0;j<3;j++)
		if(dogs[i].best_times[j] < best_times[j])
		    best_times[j] = dogs[i].best_times[j];
	}

    for(j=0;j<3;j++)
	if(attribute == (j + 2) * 10)
	    return best_times[j];

    if(attribute == 50)
	return points;
    else if(attribute == 60)
	return wins;
    else if(attribute == 90)
	return (best_times[0] * 2 + best_times[1] +
		best_times[2] * 0.666666) / 3;
    else if(attribute == 100)
    {
	if(races > 0)
	    return points / races;
	else
	    return 0;
    }
    else if(attribute == 110)
    {
	if(races > 0)
	    return wins / races;
	else
	    return 0;
    }
    else if(attribute == 200)
	return races;

    return -1;
}

/* monthly check of the fastest times 
   and most points/wins records */
gint check_records(void)
{
    gint i, j;

    /* check dog records */
    for(i=0;i<400;i++)
	if(dogs[i].age > -1)
	{
	    for(j=0;j<3;j++)
		if(dogs[i].best_times[j + 3] < dog_records[j].value)
		    write_dog_record(&dog_records[j],
				     i, (j + 2) * 10);
	    if(points_of(dogs[i]) > dog_records[3].value)
		write_dog_record(&dog_records[3], i, 50);
	    if(dogs[i].races[1] > dog_records[5].value)
		write_dog_record(&dog_records[5],
				 i, 60);
	}

    /* check kennel records */
    for(i=0;i<20;i++)
    {
	if(kennel_best(i, 90) < kennel_records[0].value)
	    write_kennel_record(&kennel_records[0], i,
				kennel_best(i, 90) );
	if(kennel_best(i, 50) > kennel_records[1].value)
	    write_kennel_record(&kennel_records[1], i,
				kennel_best(i, 50));
	if(kennel_best(i, 60) > kennel_records[3].value)
	    write_kennel_record(&kennel_records[3], i,
				kennel_best(i, 60) );
    }
    
    return 0;
}

/* write a new dog record */
gint write_dog_record(dog_record *record, gint id, gint type)
{
    gint i;
    
    /* some records are
       checked in the first week of every new
       season, so: */
    if(week == 1 && month == 1 && season > 1)
	record->season = season - 1;
    else
	record->season = season;
    
    strcpy(record->name, dogs[id].name);
    strcpy(record->kennel_name,
	   kennels[(gint)((gfloat)id / 20)].name);
   
    for(i=0;i<3;i++)
	if(type == (i + 2) * 10)
	    record->value = dogs[id].best_times[i + 3];

    if(type == 50)
	record->value = points_of(dogs[id]);
    else if(type == 60)
	record->value = dogs[id].races[1];
    else if(type == 100)
	record->value = (gfloat)points_of(dogs[id]) / (gfloat)dogs[id].races[0];
    else if(type == 110)
	record->value = (gfloat)dogs[id].races[1] / (gfloat) dogs[id].races[0];

    return 0;
}

/* write a new kennel record */
gint write_kennel_record(kennel_record *record, gint id, gfloat value)
{
    if(season == 0)
	record->season = 1;
    else
	record->season = season;

    strcpy(record->name, kennels[id].name);
    record->value = value;
    
    return 0;
}

/* return the id of the first 'invisible' dog
   in the kennel with id 'id'; if the kennel's
   full, return -1; */
gint first_free_place_in_kennel(gint id)
{
    gint i;

    for(i= id * 20; i < (id + 1) * 20; i++)
	if(dogs[i].age == -1)
	    return i;

    return -1;
}

/* see if there was fighting in the human player's kennel */
gint fighting(void)
{
    gint i;
    gint number_of_injured;
    gint dog_injured;
    gint safety_counter = 500;
    gfloat fighting_probability = 0.05;
    
    /* each bitch in heat increases the probability */
    for(i=0;i<20;i++)
	if(dogs[i].age > -1 && dogs[i].status == 0)
	    fighting_probability += 0.03;

    if(rnd(0,1) < fighting_probability)
    {
	number_of_injured = (gint)rint( rnd(1,3) );
	
	for(i=0;i<number_of_injured;i++)
	{
	    dog_injured = (gint)rint( rnd(0,19) );
	    while( (dogs[dog_injured].age == -1 ||
		    dogs[dog_injured].status == 0 ||
		    dogs[dog_injured].status > 19) &&
		   safety_counter > 0)
	    {
		dog_injured = (dog_injured + 1) % 20;
		safety_counter--;
	    }

	    dogs[dog_injured].status = 20;
	}
	
	return 2;
    }
    
    return 0;
}

